class ClientDataSet {
  constructor(dataset) {
    this.data = [];
    this.point = 0;
    this.eof = true;
    if (!!dataset) {
      this.setData(dataset);
    }
  }

  defineEof() {
    this.eof = !this.data.length | (this.point == this.data.length - 1);
  }

  setData(dataset) {
    if (!!dataset) {
      this.data = typeof dataset == 'string' ? JSON.parse(dataset) : dataset;
      this.point = 0;
    } else {
      this.data = [];
      this.point = 0;
      this.defineEof();
    }
  }

  getData() {
    return this.data;
  }

  first() {
    this.point = 0;
    this.current = this.data[this.point];
    this.defineEof();
  }

  next() {
    if (!this.eof) {
      this.point++;
      this.current = this.data[this.point];
      this.defineEof();
    }
  }

  prior() {
    if (!!this.point) {
      this.point--;
      this.current = this.data[this.point];
      this.defineEof();
    }
  }

  open() {}

  close() {}

  findIndex(fields, values) {
    return this.data.findIndex((element) => {
      let mismatch = 0;

      fields.forEach((field, i) => {
        if (element[field] !== values[i]) {
          mismatch += 1;
        }
      });

      return !mismatch;
    });
  }

  delete(fields, values) {
    const index = this.findIndex(fields, values);

    return this.data.splice(index, 1);
  }

  locate(fields, values) {
    if (fields.length && values.length) {
      const index = this.findIndex(fields, values);

      return index < 0 ? null : this.data[index];
    }

    return null;
  }
}

module.exports = ClientDataSet;
