const io = require('socket.io');
const AuthService = require('./AuthService');
const UpService = require('./UpService');
const adqGetDashboardFiliais = require('../model/adqGetDashboardFiliais');
const deliveryController = require('../controller/DeliveryController');
const saleController = require('../controller/SaleController');
const sefazController = require('../controller/SefazController');
const intervalMonitoring = require('../functions/IntervalMonitoring');
const paginacao = require('../functions/Paginacao');
const updaterController = require('../controller/UpdaterController');

class SocketService {
  async start(http) {
    this.clientlist = {};
    this.browserlist = {};
    this.slavelist = await this.getDashboardFiliais();
    this.defaultInitTimeout = 5000;

    this.parametersError = [];
    this.vendasError = [];
    this.satError = [];
    this.io = io(http, { cors: '*' });

    this.io.on('connection', (socket) => {
      this.socket = socket;
      console.log('newconnection: ', socket.id);
      socket.emit('updateVersion', { serviceName: 'xxx' });

      socket.on('login', (data) => {
        this.login(socket, data);
      });

      /**
       * Exportação de vendas
       */
      socket.on('iniciaExportacao', (data) => {
        try {
          data.payload = JSON.parse(data.payload);
          UpService.setInsereServidor(data.CDFILIAL, data.CDCAIXA, data.CDLOJA, data.payload);
        } catch (e) {}
      });

      socket.on('importar', (data) => {
        console.log('importar', data);
        let target = this.slavelist[socket.email][data.CDFILIAL];
        this.io
          .to(target.id)
          .emit('command', { type: 'import', data: { dataUrl: '/API/getStructure' } });
      });

      socket.on('confirmation', (data) => {
        // console.log(data);
      });

      socket.on('disconnect', () => {
        const subsidiaryIndex = this.slavelist.findIndex((item) => {
          return item.socketId === socket.id;
        });

        if (this.slavelist[subsidiaryIndex]) {
          this.slavelist[subsidiaryIndex].situacao = 'Inativa';

          const portalList = Object.keys(this.clientlist).map((item) => {
            return this.clientlist[item].id;
          });
          if (portalList.length) {
            this.io.to(portalList).emit('login', {
              text: 'UNIDADE DESCONECTADA',
              error: false,
              message: {
                data: this.slavelist[subsidiaryIndex],
              },
            });
          }
        }
      });

      /**
       * Socket de Vendas
       */
      socket.on('sales', (data) => {
        // Executar a primeira vez mais rápido quando abrir o Grid
        setTimeout(() => {
          doSales(this.vendasError, this.io);
        }, this.defaultInitTimeout);
        // Ficar executando N vezes para o monitoramento
        setInterval(() => {
          doSales(this.vendasError, this.io);
        }, intervalMonitoring(data));
      });

      function doSales(vendasError, io) {
        saleController.salesSlave(vendasError).then((response) => {
          io.to(socket.id).emit('sales', {
            text: 'VENDAS',
            error: false,
            message: {
              data: response,
            },
          });
        }).catch((error) => {
          console.log(error);
        });
      }

      /**
       * Socket de Delivery
       */
      socket.on('salesDelivery', (data) => {
        // Executar a primeira vez mais rápido quando abrir o Grid
        setTimeout(() => {
          doSalesDelivery(data, this.io);
        }, this.defaultInitTimeout);
        // Ficar executando N vezes para o monitoramento
        setInterval(() => {
          doSalesDelivery(data, this.io);
        }, intervalMonitoring(data));
      });

      function doSalesDelivery(data, io) {
        deliveryController.getAssistantDeliveryList(data).then((response) => {
        /** Lista de vendas Delivery */
          io.to(socket.id).emit('salesDelivery', {
            text: 'DELIVERY',
            error: false,
            message: {
              data: response,
            },
          });
        }).catch((error) => {
          console.log('Erro foi: ', error);
        });
      }

      /**
       * Socket de Sat
       */
      socket.on('sat', (data) => {
        /** Conectar com a Loja */
        if (data.socketClient !== undefined) {
          this.io.to(data.socketClient).emit('satSlave', data);
        }

        // Executar a primeira vez mais rápido quando abrir o Grid
        setTimeout(() => {
          doSat(this.io, this.satError);
        }, this.defaultInitTimeout);
        // Ficar executando N vezes para o monitoramento
        setInterval(() => {
          doSat(this.io, this.satError);
        }, intervalMonitoring(data));
      });

      function doSat(io, satError) {
        io.to(socket.id).emit('sat', {
          text: 'SAT',
          error: false,
          message: {
            data: satError,
          },
        });
      }

      /**
       * Socket Update
       */
      socket.on('updater', (data) => {
        /** Conectar com a Loja */
        // if (data.socketClient !== undefined) {
        //   this.io.to(data.socketClient).emit('satSlave', data);
        // }

        updaterController.getUnidades().then((result) => {
          const newItens = [];
          result.forEach((results) => {
            updaterController.getVersaoAtual(results.IDUPDATERSISTEMAS).then((resultss) => {
              results.VERSAOATUAL = resultss;
              newItens.push(results);
            });
          });
          setInterval(() => {
            this.io.to(socket.id).emit('updater', {
              text: 'UPDATE',
              error: false,
              message: {
                data: newItens,
              },
            });
          }, intervalMonitoring(data));
        }).catch((err) => {
          console.log(err);
        });
      });

      socket.on('updaterHistorico', (data) => {
        updaterController.getHistorico(data).then((result) => {
          this.io.to(socket.id).emit('updaterHistorico', {
            text: 'UPDATE',
            error: false,
            message: {
              data: result,
            },
          });
        });
      });

      socket.on('updaters', (data) => {
        if (data.tipo === 'up') {
          updaterController.updaterSistemas(data).then((result) => {
            console.log(result);
          }).catch((err) => {
            console.log(err);
          });
        } else if (data.tipo === 'down') {
          updaterController.getUnidades().then((result) => {
            console.log(result);
          }).catch((err) => {
            console.log(err);
          });
        } else if (data.tipo === 'historico') {
          updaterController.getUnidades().then((result) => {
            console.log(result);
          }).catch((err) => {
            console.log(err);
          });
        } else if (data.tipo === 'install') {
          updaterController.getUnidades().then((result) => {
            console.log(result);
          }).catch((err) => {
            console.log(err);
          });
        }
      });

      /**
       * Socket de Parametros
       */
      socket.on('parameters', (data) => {
        // Executar a primeira vez mais rápido quando abrir o Grid
        setTimeout(() => {
          doParameters(this.io, this.parametersError);
        }, this.defaultInitTimeout);
        // Ficar executando N vezes para o monitoramento
        setInterval(() => {
          doParameters(this.io, this.parametersError);
        }, intervalMonitoring(data));
      });

      function doParameters(io, parametersError) {
        io.to(socket.id).emit('parameters', {
          text: 'PARAMETERS',
          error: false,
          message: {
            data: parametersError,
          },
        });
      }

      /**
       * Socket de Sefaz
       */
      socket.on('sefaz', (data) => {
        // Executar a primeira vez mais rápido quando abrir o Grid
        setTimeout(() => {
          doSefaz(data, this.io);
        }, this.defaultInitTimeout);
        // Ficar executando N vezes para o monitoramento
        setInterval(() => {
          doSefaz(data, this.io);
        }, intervalMonitoring(data));
      });

      function doSefaz(data, io) {
        sefazController.getSefaz(data).then((response) => {
          io.to(socket.id).emit('sefaz', {
            text: 'SEFAZ',
            error: false,
            message: {
              data: response,
            },
          });
        }).catch((error) => {
          console.log('Erro foi: ', error);
        });
      }
    });
  }

  import(socket, data) {}

  login(socket, data) {
    // Não é uma filial que está conectando e sim um portal
    if (!data.CDFILIAL) {
      // TODO Autenticar esse cliente, senão desconectar ele

      let auth = AuthService.autenticar(data.email, data.password)
        .then((result) => {
          // Se estiver autenticado coloca ele na lista simples de controle
          socket.email = data.email;

          this.clientlist[data.email] = socket;
          this.io
            .to(socket.id)
            .emit('confirmLogin', { error: false, message: 'Você está conectado', result: result });

          if (!data.slave) {
            if (this.slavelist === undefined) {
              this.getDashboardFiliais(data).then((dashboard) => {
                this.io.to(socket.id).emit('login', {
                  text: 'PORTAL CONECTADO',
                  error: false,
                  message: dashboard,
                  result,
                });
              });
            } else {
              this.io.to(socket.id).emit('login', {
                text: 'PORTAL CONECTADO',
                error: false,
                message: this.slavelist,
                result,
              });
            }
          }
        })
        .catch((error) => {
          this.io.to(socket.id).emit('message', 'Você não é um usuário autorizado!');
          socket.disconnect();
        });
    } else {
      // Uma filial se coneta ao master
      // TODO Autenticar esse cliente, senão desconectar ele
      let auth = AuthService.autenticar(data.email, '', data.CDFILIAL)
        .then((result) => {
          // Se estiver autenticado coloca ele na lista simples de controle
          socket.email = data.email;
          socket.CDFILIAL = data.CDFILIAL;

          const subsidiaryIndex = this.slavelist.findIndex((item) => {
            return item.codigo === data.CDFILIAL;
          });

          if (this.slavelist[subsidiaryIndex]) {
            this.slavelist[subsidiaryIndex].situacao = 'Ativo';
            this.slavelist[subsidiaryIndex].socketId = socket.id;

            const portalList = Object.keys(this.clientlist).map((item) => {
              return this.clientlist[item].id;
            });

            this.io.to(socket.id).emit('message', result);
            this.io
              .to(socket.id)
              .emit('command', { type: 'downloadFromUrl', data: { dataUrl: '/API/getStructure' } });
            // this.io.to(socket.id).emit('consultaSAT', {});

            if (portalList.length) {
              this.io.to(portalList).emit('login', {
                text: 'UNIDADE CONECTADA',
                error: false,
                message: {
                  data: this.slavelist,
                },
                result,
              });
            }

            /**
             * Monitoramento das vendas
             * Faz a solicitação para o Slaver
            */
            this.io.to(portalList).emit('saleSlave', data);
            socket.on('saleSlaveError', (vendasError) => {
              this.vendasError = vendasError;
            });

            /**
             * Monitoramento dos Sats
             * Faz a solicitação para o Slaver
            */
            if (!this.slavelist[subsidiaryIndex].sat == 'NOT_SAT') {
              this.io.to(portalList).emit('satSlave', data);
              socket.on('satSlaveError', (satError) => {
                this.satError = satError;
              });
            }

            /**
             * Monitoramento dos parametros
             * parameters
             */
            this.io.to(portalList).emit('parametersSlave', data);
            socket.on('parametersSlaveError', (parametersError) => {
              this.parametersError = parametersError;
            });
          }
        })
        .catch((error) => {
          this.io.to(socket.id).emit('message', 'Você não é um usuário autorizado!');
          socket.disconnect();
        });
    }
  }

  async getDashboardFiliais(data = null) {
    const filiais = [];
    const { rows } = await adqGetDashboardFiliais.get(paginacao(data));
    rows.forEach((element) => {
      filiais.push({
        id: element[0],
        codigo: element[0],
        name: element[1],
        situacao: 'Inativa',
        email: '',
        socketId: '',
        sat: element[2] == 0 ? 'NOT_SAT' : false,
        vendas: false,
        delivery: false,
        websocket: '',
      });
    });

    if (this.slavelist !== undefined)
    {
      filiais.concat(this.slavelist);
    }

    return filiais;
  }
}

module.exports = new SocketService();
