import { SocketEvent } from '@/socket/socket-event';
import {
  Config,
  IDictionary,
  IEventParam,
  Router,
} from '@zeedhi/core';
import { TekGrid } from '@zeedhi/teknisa-components-common';
import { io, Socket } from 'socket.io-client';

export class SimpleGridController {
  public socket?: Socket;

  /**
   * Método executado quando o grid é criado
   */
  public gridMounted({ component }: IEventParam<TekGrid>) {
    // inicializa o socket
    this.socket = io(Config.endPoint);

    // registra um listener para o evento de connection
    this.socket.off(SocketEvent.CONNECTION)
      .on(SocketEvent.CONNECTION, (employees: IDictionary[]) => {
        component.datasource.updateData(employees);
        console.warn(employees);
      });

    // registra um listener para o evento employee-changed
    this.socket.off(SocketEvent.EMPLOYEE_CHANGED)
      .on(SocketEvent.EMPLOYEE_CHANGED, async (employee: IDictionary) => {
        await component.datasource.put(employee);
        await component.datasource.get();
      });
  }

  /**
   * Método que recebe a filial para a navegação
   */
  public ids(obj:any) {
    console.warn(obj);
  }

  public push() {
    Router.push('/', this.ids);
  }

  /**
   * Método executado logo antes do grid ser destruído
   */
  public gridBeforeDestroy() {
    this.socket?.disconnect();
  }
}
