import { Modal, ModalService } from '@zeedhi/common';
import { SocketEvent } from '@/socket/socket-event';
import {
  Config,
  IDictionary,
  IEventParam,
} from '@zeedhi/core';
// import { TekGrid } from '@zeedhi/teknisa-components-common';
import { ZdTreeGrid } from '@zeedhi/vuetify';
import { io, Socket } from 'socket.io-client';

export class UpdatesController {
  public socket?: Socket;

  public title: any;

  public visibleCondition({ row }: IDictionary) {
    if (row.UPSISVERATUAL === '-' || row.UPSISVERATUAL <= '0.0.1') {
      return false;
    }
    return true;
  }

  public visibleConditionUpdate({ row }: IDictionary) {
    let up = false;
    if (
      row.UPSISVERATUAL === '-'
      || row.UPSISVERDISPONIVEL.length === 0
      || row.UPSISVERDISPONIVEL === undefined
      ) {
      up = false;
    } else {
      up = true;
    }

    if (row.VERSAOATUAL.UPVERSTATUS === 0) {
      up = false;
    }
    return up;
  }

  public visibleConditionDowngrade({ row }: IDictionary) {
    if (row.UPSISVERATUAL === '-' || row.UPSISVERATUAL <= '0.0.1') {
      return false;
    }
    if (row.VERSAOATUAL.UPVERSTATUS === 0) {
      return false;
    }
    return true;
  }

  public visibleConditionStatus({ row }: IDictionary) {
    let cor;
    [row].forEach((element:any) => {
      if (
        element.UPSISVERATUAL === '-'
        || element.UPSISVERDISPONIVEL !== ''
      ) {
        cor = 'red';
      } else {
        cor = 'green';
      }
      if (element.VERSAOATUAL.UPVERSTATUS === 0) {
        cor = 'gray';
      }
    });

    return cor;
  }

  // public startModal() {
  //   Loader.getInstance('HistoricoController').createFullscreenModal();
  // }

  private fullscreenModal!: Modal;

  public async createFullscreenModal() {
    this.fullscreenModal = await ModalService.createFromJson('historico', true);
  }

  public destroyFullscreenModal() {
    this.fullscreenModal?.destroy();
  }

  public openFullscreenModal() {
    this.fullscreenModal.show();
  }

  public closeFullscreenModal() {
    this.fullscreenModal.hide();
  }

  /**
   * Método executado quando o grid é criado
   */
  public gridMounted({ component }: IEventParam<ZdTreeGrid>) {
    // inicializa o socket
    this.socket = io(Config.endPoint);

    this.socket.off(SocketEvent.CONNECTION)
      .emit('login', {
        email: 'francismar.araujo@teknisa.com',
        password: 'nineninenine',
        slave: false,
      });

    this.socket.off(SocketEvent.CONNECTION)
      .on('confirmLogin', async (branc: IDictionary) => {
        if (branc) {
          console.warn();
        }
      });

    this.socket.off(SocketEvent.CONNECTION)
      .emit('updater', {
        slave: false,
      });

    this.socket.off(SocketEvent.CONNECTION)
      .on('updater', async (branc: IDictionary) => {
        await component.datasource.updateData(branc.message.data);
        await component.datasource.get();
      });
  }

  public selectClick({ component }: any) {
    const itemsText = component.selectedRows.map((item: any) => item.id).join(', ');
    component.getChildInstance('selected-rows').text = `selectedRows = [${itemsText}]`;
  }

  /**
   * @author daniel.souza
   * @param element elemento clicado pelo usuário
   */
  public updaters(element:any) {
    this.socket = io(Config.endPoint);
    this.socket.off(SocketEvent.CONNECTION)
      .emit('updaters', {
        slave: false,
        dados: {
          id: element.row.id,
          IDUPDATER: element.row.IDUPDATER,
          CDFILIAL: element.row.CDFILIAL,
          NOMEFILIAL: element.row.NOMEFILIAL,
          IDUPDATERSISTEMAS: element.row.IDUPDATERSISTEMAS,
          DEVOPSID: element.row.DEVOPSID,
          APLICACAO: element.row.APLICACAO,
          SERVICO: element.row.SERVICO,
          UPSISVERATUAL: element.row.UPSISVERATUAL,
          SISTEMASSTATUS: element.row.SISTEMASSTATUS,
        },
        operador: localStorage.getItem('CDOPERADOR'),
        tipo: element.column.name,
      });

    this.socket.off(SocketEvent.CONNECTION)
      .on('updaters', async (branc: IDictionary) => {
        console.warn('Branch:', branc);
      });
  }

  /**
   * Método executado logo antes do grid ser destruído
   */
  public gridBeforeDestroy() {

  }
}
